@@ -1,5 +1,7 @@
 from .app import (
     Jembe,
+    Processor,
+    get_jembe,
     get_storage,
     get_storages,
     get_private_storage,
@@ -8,9 +10,10 @@ from .app import (
 )
 from .component import Component, ComponentReference
 from .common import JembeInitParamSupport, ComponentRef, DisplayResponse
-from .component_config import action, listener, redisplay, config, UrlPath, ComponentConfig, RedisplayFlag
+from .component_config import config, action, listener, redisplay, UrlPath, ComponentConfig, RedisplayFlag
 from .processor import Event
 from .exceptions import (
+    ComponentPreviousStateUnavaiableError,
     BadRequest,
     Unauthorized,
     Forbidden,
@@ -20,14 +23,15 @@ from .exceptions import (
     Gone,
     NotImplemented,
     InternalServerError,
+    JembeError
 )
 from .files import File, Storage, DiskStorage
 from .utils import page_url, run_only_once
 
-__version__ = "0.3.0"
-
 __all__ = (
     "Jembe",
+    "Processor",
+    "get_jembe",
     "get_storage",
     "get_storages",
     "get_private_storage",
@@ -46,6 +50,7 @@ __all__ = (
     "ComponentRef",
     "DisplayResponse",
     "Event",
+    "ComponentPreviousStateUnavaiableError",
     "BadRequest",
     "Unauthorized",
     "Forbidden",
@@ -55,6 +60,7 @@ __all__ = (
     "Gone",
     "NotImplemented",
     "InternalServerError",
+    "JembeError",
     "File",
     "Storage",
     "DiskStorage",
