@@ -1,23 +1,23 @@
 from typing import (
-    Iterable,
     TYPE_CHECKING,
+    Iterable,
     Optional,
     Union,
     Dict,
     Any,
     List,
     Tuple,
-    get_args,
-    get_origin,
     Type,
 )
-from collections.abc import Sequence as collectionsSequence
+import re
 from urllib.parse import quote_plus
 from functools import cached_property
 from copy import deepcopy, copy
 from abc import ABCMeta
-from inspect import Parameter, isclass, signature, getmembers, Signature
-from .exceptions import JembeError, NotFound
+from inspect import Parameter, signature, getmembers, Signature
+
+from flask.json import dumps
+from .exceptions import JembeError, NotFound, ComponentPreviousStateUnavaiableError
 from flask import render_template, render_template_string, current_app
 from markupsafe import Markup
 from .component_config import ComponentConfig
@@ -27,20 +27,20 @@ from .processor import (
     CallDisplayCommand,
     InitialiseCommand,
     EmitCommand,
-    Processor,
 )
 from .common import (
-    JembeInitParamSupport,
     exec_name_to_full_name,
-    get_annotation_type,
-    DisplayResponse,
+    load_param,
+    dump_param,
 )
-from .files import Storage
+
+if TYPE_CHECKING:
+    import jembe
 
 
 class ComponentState(dict):
     """
-    Two instance of the same compoennt, component with same full_name, in the same
+    Two instance of the same component, component with same full_name, in the same
     state should behawe identically.
 
     State of the component is defined by state params, __init__ params whose name
@@ -56,7 +56,9 @@ class ComponentState(dict):
 
     def __setitem__(self, key, value):
         if not key in self.keys():
-            raise JembeError("Can't add new param to component state")
+            raise JembeError(
+                "Can't add new param '{}' to component state {}".format(key, self)
+            )
         return super().__setitem__(key, value)
 
     def __delitem__(self, key):
@@ -87,7 +89,9 @@ class ComponentState(dict):
         return c
 
     def tojsondict(
-        self, component_class: Union[Type["Component"], "Component"], full=False
+        self,
+        component_class: Union[Type["jembe.Component"], "jembe.Component"],
+        full=False,
     ):
         return {
             k: copy(component_class.dump_init_param(k, v))
@@ -102,7 +106,7 @@ class ComponentReference:
     usage for link:
     lamda self: self.component('name')[.jrl...]
     lambda self: self.component('/main').component('name')[.jrl|url|is_accessible]
-    usage for displaying compoennt in template:
+    usage for displaying component in template:
     {{component('name')}}
     {{component('name').component('name')}} -- shuld raise exception
     usage in template to display link
@@ -136,16 +140,24 @@ class ComponentReference:
                         caller_exec_name,
                         pname,
                         {},
-                        merge_existing_params,
+                        # merge_existing_params,
                     )
                 else:
                     cr = cr.component(pname)
-            cr = (
-                cr.component(name_split[-1], **kwargs)
-                if cr is not None
-                else cls(
+            if cr is not None:
+                cr = (
+                    cr.component(name_split[-1], **kwargs)
+                    if merge_existing_params
+                    else cr.component_reset(name_split[-1], **kwargs)
+                )
+            else:
+                cr = cls(
                     caller_exec_name, name_split[-1], kwargs, merge_existing_params
                 )
+        elif "/" not in name and "." not in name and "()" in name:
+            # compoent("cancel()").is_accessible
+            cr = cls(caller_exec_name, ".", dict(), merge_existing_params).call(
+                name.replace("()", ""), **kwargs
             )
         else:
             cr = cls(caller_exec_name, name, kwargs, merge_existing_params)
@@ -176,7 +188,7 @@ class ComponentReference:
         self.action = ComponentConfig.DEFAULT_DISPLAY_ACTION
         self.action_args: Tuple[Any, ...] = ()
         self.action_kwargs: dict = {}
-        self.kwargs = kwargs
+        self.kwargs = {k: v for k, v in kwargs.items() if not k.startswith("_")}
         self.merge_existing_params = merge_existing_params
 
         self._component_initialise_done = False
@@ -204,7 +216,7 @@ class ComponentReference:
 
     def _component(
         self, merge_existing_params, jmb_exec_name: str, **kwargs
-    ) -> "ComponentReference":
+    ) -> "jembe.ComponentReference":
         cr = ComponentReference(
             self.exec_name, jmb_exec_name, kwargs, merge_existing_params
         )
@@ -219,7 +231,7 @@ class ComponentReference:
         cr.base_jrl = self.jrl
         return cr
 
-    def component(self, jmb_exec_name: str, **kwargs) -> "ComponentReference":
+    def component(self, jmb_exec_name: str, **kwargs) -> "jembe.ComponentReference":
         if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION:
             raise JembeError(
                 "<{}> Cant reference child component when action '{}' is called".format(
@@ -228,7 +240,9 @@ class ComponentReference:
             )
         return self._component(True, jmb_exec_name, **kwargs)
 
-    def component_reset(self, jmb_exec_name: str, **kwargs) -> "ComponentReference":
+    def component_reset(
+        self, jmb_exec_name: str, **kwargs
+    ) -> "jembe.ComponentReference":
         return self._component(False, jmb_exec_name, **kwargs)
 
     def _init_component(self):
@@ -248,7 +262,7 @@ class ComponentReference:
             )
 
     @property
-    def component_instance(self) -> Optional["Component"]:
+    def component_instance(self) -> Optional["jembe.Component"]:
         if self.name == "." and self.root_renderer == self:
             return self.processor.components[self.exec_name]
         else:
@@ -269,7 +283,11 @@ class ComponentReference:
                     )
                 )
                 return True
-            return self.action not in ci._jembe_disabled_actions
+
+            return (
+                self.action in ci._config.component_actions
+                and self.action not in ci._jembe_disabled_actions
+            )
         else:
             if self.name == ".":
                 return True
@@ -298,27 +316,13 @@ class ComponentReference:
     def jrl(self) -> str:
         if not self.is_accessible:
             raise NotFound()
-
-        def _prep_v(v):
-            if isinstance(v, bool):
-                return "true" if v else "false"
-            elif isinstance(v, (int, float)):
-                return v
-            return "'{}'".format(v)
-
         jrl = (
             "component{reset}('{name}'{kwargs}{key})".format(
                 reset="_reset" if not self.merge_existing_params else "",
                 name=self.name,
                 key=",key='{}'".format(self._key) if self._key else "",
-                kwargs=",{{{}}}".format(
-                    ",".join(
-                        (
-                            "{}:{}".format(k, _prep_v(v))
-                            for k, v in self.kwargs.items()
-                            if not k.startswith("_")
-                        )
-                    )
+                kwargs=",{}".format(
+                    re.sub('(?<!\\\\)"', "'", dumps(self.kwargs, separators=(",", ":")))
                 )
                 if self.kwargs
                 else "",
@@ -328,15 +332,18 @@ class ComponentReference:
         )
         jrl += (
             # ".call('{name}',{{{kwargs}}},[{args}])".format(
-            ".call('{name}',{{{kwargs}}})".format(
+            ".call('{name}'{kwargs})".format(
                 name=self.action,
                 # args=",".join((_prep_v(v) for v in self.action_args)),
-                kwargs=",".join(
-                    (
-                        "{}:{}".format(k, _prep_v(v))
-                        for k, v in self.action_kwargs.items()
+                kwargs=",{}".format(
+                    re.sub(
+                        '(?<!\\\\)"',
+                        "'",
+                        dumps(self.action_kwargs, separators=(",", ":")),
                     )
-                ),
+                )
+                if self.action_kwargs
+                else "",
             )
             if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION
             else ".display()"
@@ -373,7 +380,7 @@ class ComponentReference:
             )
 
     @cached_property
-    def processor(self) -> Processor:
+    def processor(self) -> "jembe.Processor":
         return get_processor()
 
     def __call__(self) -> str:
@@ -384,6 +391,12 @@ class ComponentReference:
         will it raise exception (like NotFound, Forbidden, Unauthorized etc.)
         so that we can decide how to render template
         """
+        if self.exec_name in self.processor.components_marked_for_removal:
+            raise JembeError(
+                "Cant display component '{}' marked for removal in parent component template!".format(
+                    self.exec_name
+                )
+            )
         self.processor.add_command(
             InitialiseCommand(self.exec_name, self.kwargs, self.merge_existing_params),
             end=True,
@@ -395,7 +408,11 @@ class ComponentReference:
         # with appropriate template
         if self.action == ComponentConfig.DEFAULT_DISPLAY_ACTION:
             self.processor.add_command(
-                CallDisplayCommand(self.exec_name, not self.merge_existing_params),
+                CallDisplayCommand(
+                    self.exec_name,
+                    not self.merge_existing_params,
+                    displayed_by_exec_name=self.caller_exec_name,
+                ),
                 end=True,
             )
         else:
@@ -412,11 +429,11 @@ class ComponentReference:
             '<template jmb-placeholder="{}"></template>'.format(self.exec_name)
         )
 
-    def key(self, key: str) -> "ComponentReference":
+    def key(self, key: str) -> "jembe.ComponentReference":
         self._key = key
         return self
 
-    def call(self, action: str, *args, **kwargs) -> "ComponentReference":
+    def call(self, action: str, *args, **kwargs) -> "jembe.ComponentReference":
         self.action = action
         self.action_args = args
         self.action_kwargs = kwargs
@@ -425,6 +442,11 @@ class ComponentReference:
     def __html__(self):
         return self.__call__()
 
+    def if_accessible(self) -> str:
+        if self.is_accessible:
+            return self.__html__()
+        return ""
+
 
 def component(
     jmb_exec_name: str, jmb_reset: bool = True, **kwargs
@@ -525,6 +547,7 @@ class Component(metaclass=ComponentMeta):
         _component_exec_name: str,
         _jembe_injected_params_names: List[str],
         _jembe_merged_existing_params: bool,
+        _jembe_existing_component_state: Optional["ComponentState"],
         **init_params
     ):
         """
@@ -532,13 +555,18 @@ class Component(metaclass=ComponentMeta):
         because _config should be avaiable in __init__
         """
         component: "Component" = object.__new__(cls)
+        component._jembe_component_initialising = True
         component._config = _config  # type: ignore
         component._jembe_injected_params_names = _jembe_injected_params_names
         component._jembe_merged_existing_params = _jembe_merged_existing_params
+        component._jembe_existing_component_state = _jembe_existing_component_state
         component._jembe_disabled_actions = []
         component.exec_name = _component_exec_name
         component.__init__(**init_params)  # type: ignore
+        # init is called after __init__ so it can access all state variables regardles if __init__ is overwriten
         component.init()
+        component._jembe_component_initialising = None
+        component._jembe_existing_component_state = None
         return component
 
     _jembe_init_signature: "Signature"
@@ -548,7 +576,9 @@ class Component(metaclass=ComponentMeta):
     _jembe_injected_params_names: List[str]
     _jembe_config_init_params: Dict[str, Any]
     _jembe_merged_existing_params: bool
+    _jembe_existing_component_state: Optional["ComponentState"]
     _jembe_inject_into_overriden: bool
+    _jembe_component_initialising: Optional[bool]
     _config: "Config"
 
     state: "ComponentState"
@@ -572,7 +602,18 @@ class Component(metaclass=ComponentMeta):
         Executed after __init__, usefull for adding logic that should be
         executed on init but without need to rewrite/list init params.
         """
-        pass
+        # raise NotFound when componet is not accessible
+        if not self.ac_check():
+            raise self._config.DEFAULT_AC_EXCEPTION()
+
+    @property
+    def previous_state(self) -> Optional["ComponentState"]:
+        if not self._jembe_component_initialising:
+            raise ComponentPreviousStateUnavaiableError(
+                "Component previous state is only avaiable during "
+                "component intialisation in __init__ and init methods."
+            )
+        return self._jembe_existing_component_state
 
     @property
     def key(self) -> str:
@@ -604,11 +645,11 @@ class Component(metaclass=ComponentMeta):
             self.__key = name_key[1]
 
     @property
-    def has_action_or_listener_executed(self) -> bool:
+    def _jembe_has_action_or_listener_executed(self) -> bool:
         return self.__has_action_or_listener_executed
 
-    @has_action_or_listener_executed.setter
-    def has_action_or_listener_executed(self, value: bool):
+    @_jembe_has_action_or_listener_executed.setter
+    def _jembe_has_action_or_listener_executed(self, value: bool):
         if value != True:
             raise JembeError(
                 "Cannot reset action or listener execution status on component"
@@ -660,51 +701,18 @@ class Component(metaclass=ComponentMeta):
         so that it can be jsonified
 
         """
-
-        def _dump_supported_types(value, annotation):
-            atype, is_optional = get_annotation_type(annotation)
-            try:
-                if atype == set or get_origin(atype) == set:
-                    return None if (is_optional and value is None) else list(value)
-                if atype == list or get_origin(atype) == list:
-                    # TODO support tuple, dict etc.
-                    el_annotation = get_args(atype)[0]
-                    return (
-                        None
-                        if (is_optional and value is None)
-                        else list(
-                            _dump_supported_types(v, el_annotation) for v in value
-                        )
-                    )
-                elif (
-                    atype == JembeInitParamSupport
-                    or (isclass(atype) and issubclass(atype, JembeInitParamSupport))
-                    or (
-                        isclass(get_origin(atype))
-                        and issubclass(get_origin(atype), JembeInitParamSupport)
-                    )
-                    or isinstance(value, JembeInitParamSupport)
-                ):
-                    return (
-                        None
-                        if (is_optional and value is None)
-                        else atype.dump_init_param(value)
-                    )
-            except Exception as e:
-                raise ValueError(e)
-            return value
-
         if name in cls._jembe_init_signature.parameters:
             try:
                 param_hint = cls._jembe_init_signature.parameters[name]
                 if param_hint.annotation == Parameter.empty:
                     raise ValueError("Parameter without annotation")
-                return _dump_supported_types(value, param_hint.annotation)
-            except ValueError as e:
+                return dump_param(param_hint.annotation, value)
+            except Exception as e:
                 if current_app.debug or current_app.testing:
                     raise JembeError(
                         "State param {} of {}.{} with hint {} is not supported for json dump/load "
-                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param. ({}) ".format(
+                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param.\n ({})\n\n "
+                        "Please double check that you are using actuall class in annotation and not string with class name.".format(
                             name,
                             cls.__module__,
                             cls.__name__,
@@ -727,72 +735,17 @@ class Component(metaclass=ComponentMeta):
               debug mode exception will be raised (No exception will be raised in production
               because hint checking can be expensive)
         """
-
-        def _load_supported_types(value, annotation):
-            """ returns loaded value or raise ValueError"""
-            # TODO add support for multiple annotation types Union[a,b,c] etc
-
-            atype, is_optional = get_annotation_type(annotation)
-            try:
-                if atype == bool:
-                    return None if (is_optional and value is None) else bool(value)
-                elif atype == int:
-                    return None if (is_optional and value is None) else int(value)
-                elif atype == str:
-                    return None if (is_optional and value is None) else str(value)
-                elif atype == float:
-                    return None if (is_optional and value is None) else float(value)
-                elif atype == dict or get_origin(atype) == dict:
-                    # TODO recursive dict
-                    return None if (is_optional and value is None) else dict(value)
-                elif atype == tuple or get_origin(atype) == tuple:
-                    # TODO recursive tuple
-                    return None if (is_optional and value is None) else tuple(value)
-                elif atype == list or get_origin(atype) == list:
-                    el_annotation = get_args(atype)[0]
-                    return (
-                        None
-                        if (is_optional and value is None)
-                        else list(
-                            _load_supported_types(v, el_annotation) for v in value
-                        )
-                    )
-                elif atype == set or get_origin(atype) == set:
-                    # TODO recursive set
-                    return None if (is_optional and value is None) else set(value)
-                elif get_origin(atype) == collectionsSequence:
-                    # TODO recursive collection
-                    return None if (is_optional and value is None) else tuple(value)
-                elif (
-                    atype == JembeInitParamSupport
-                    or (isclass(atype) and issubclass(atype, JembeInitParamSupport))
-                    or (
-                        isclass(get_origin(atype))
-                        and issubclass(get_origin(atype), JembeInitParamSupport)
-                    )
-                ):
-                    if is_optional and value is None:
-                        return None
-                    elif isinstance(value, atype):
-                        return value.load_init_param(value)
-                    else:
-                        return atype.load_init_param(value)
-            except Exception as e:
-                raise ValueError(e)
-
-            raise ValueError("Unsuported annotation type {}".format(annotation))
-
         if name in cls._jembe_init_signature.parameters:
             try:
                 param_hint = cls._jembe_init_signature.parameters[name]
                 if param_hint.annotation == Parameter.empty:
                     raise ValueError("Parameter without annotation")
-                return _load_supported_types(value, param_hint.annotation)
+                return load_param(param_hint.annotation, value)
             except ValueError as e:
                 if current_app.debug or current_app.testing:
                     raise JembeError(
                         "State param {} of {}.{} with hint {} is not supported for json dump/load "
-                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param. ({}) ".format(
+                        "nor custom encoding/decoding logic is defined in dump_init_param/load_init_param. \n\n({}) ".format(
                             name,
                             cls.__module__,
                             cls.__name__,
@@ -823,10 +776,10 @@ class Component(metaclass=ComponentMeta):
         """
         return dict()
 
-    def inject_into(self, cconfig: "ComponentConfig") -> Dict[str, Any]:
+    def inject_into(self, cconfig: "jembe.ComponentConfig") -> Dict[str, Any]:
         """
         inject_into params are used to inject  params into child component.
-        This params usually defines values required by child compoennt that are
+        This params usually defines values required by child component that are
         generated or optained by parent component.
 
         inject_into is called when child component is initialised, which can
@@ -857,7 +810,7 @@ class Component(metaclass=ComponentMeta):
     def isinjected(self, param_name: str) -> bool:
         return param_name in self._jembe_injected_params_names
 
-    def display(self) -> "DisplayResponse":
+    def display(self) -> "jembe.DisplayResponse":
         return self.render_template()
 
     def render_template(
@@ -878,6 +831,10 @@ class Component(metaclass=ComponentMeta):
             **context,
         }
         template = template if template else self._config.template
+        if not isinstance(template, str):
+            template = list(template)
+        # add reference to itself in context (usefull for passing parameters in macros)
+        context["_context"] = context
         return render_template(template, **context)
 
     def render_template_string(self, source, **context):
@@ -895,6 +852,8 @@ class Component(metaclass=ComponentMeta):
             **self._get_default_template_context(),
             **context,
         }
+        # add reference to itself in context (usefull for passing parameters in macros)
+        context["_context"] = context
         return render_template_string(source, **context)
 
     def _get_default_template_context(self) -> Dict[str, Any]:
@@ -909,6 +868,12 @@ class Component(metaclass=ComponentMeta):
         return {
             # adds state params
             **self.state,
+            # add class variables not starting with underscore
+            **{
+                name: value
+                for name, value in vars(self.__class__).items()
+                if not name.startswith("_")
+            },
             # add instance variables not starting with underscore
             **{
                 name: value
@@ -923,10 +888,12 @@ class Component(metaclass=ComponentMeta):
                     self.__class__,
                     lambda o: isinstance(o, property) or isinstance(o, cached_property),
                 )
+                if property_name != "previous_state"
             },
             # command to render subcomponents
             "component": self._jinja2_component,
             "component_reset": self._jinja2_component_reset,
+            "placeholder": self._jinja2_placeholder,
             # add helpers
             "_config": self._config,
         }
@@ -936,7 +903,7 @@ class Component(metaclass=ComponentMeta):
         kwargs: Dict[str, Any],
         name: Optional[str] = None,
         merge_existing_params: bool = True,
-    ) -> "ComponentReference":
+    ) -> "jembe.ComponentReference":
         if name is None:
             try:
                 return self.__prev_sub_component_renderer.active_renderer
@@ -951,36 +918,110 @@ class Component(metaclass=ComponentMeta):
             return self.__prev_sub_component_renderer
 
     def component(
-        self, _jmb_component_name: str = ".", **kwargs
-    ) -> "ComponentReference":
-        return self._component_reference(kwargs, _jmb_component_name)
+        self, _jembe_component_name: str = ".", **kwargs
+    ) -> "jembe.ComponentReference":
+        return self._component_reference(kwargs, _jembe_component_name)
 
     def component_reset(
         self, _jmb_component_name: str = ".", **kwargs
-    ) -> "ComponentReference":
+    ) -> "jembe.ComponentReference":
         return self._component_reference(kwargs, _jmb_component_name, False)
 
     def _jinja2_component(
         self, _jmb_component_name: Optional[str] = None, **kwargs
-    ) -> "ComponentReference":
+    ) -> "jembe.ComponentReference":
         return self._component_reference(kwargs, _jmb_component_name)
 
     def _jinja2_component_reset(
         self, _jmb_component_name: Optional[str] = None, **kwargs
-    ) -> "ComponentReference":
+    ) -> "jembe.ComponentReference":
         return self._component_reference(kwargs, _jmb_component_name, False)
 
-    def emit(self, name: str, **params) -> "EmitCommand":
+    def _jinja2_placeholder(self, _jmb_copmonent_name: str):
+        return Markup(
+            '<{dtag} jmb-placeholder-permanent="{exec_name}/{component_name}"></{dtag}>'.format(
+                exec_name=self.exec_name,
+                component_name=_jmb_copmonent_name,
+                dtag="template",
+            )
+        )
+
+    def emit(self, _jmb_event_name: str, **params) -> "EmitCommand":
         processor = get_processor()
-        emmit_command = EmitCommand(self.exec_name, name, params)
+        emmit_command = EmitCommand(self.exec_name, _jmb_event_name, params)
         processor.add_command(emmit_command)
         return emmit_command
 
-    def get_storage(self, storage_name: Optional[str] = None) -> "Storage":
+    def display_component(
+        self,
+        _jmb_component_name: str,
+        _jmb_component_key: Optional[str] = None,
+        **init_params
+    ):
+        """
+        Calls initCommand and displayComand of subcomponent without need to
+        redisplay this commponent.
+        """
+        if _jmb_component_name not in self._config.components.keys():
+            raise JembeError(
+                "Component '{}' is not valid sub component of '{}'!".format(
+                    _jmb_component_name, self._config.full_name
+                )
+            )
+        processor = get_processor()
+        cexec_name = "{}/{}".format(
+            self.exec_name,
+            _jmb_component_name
+            if _jmb_component_key is None
+            else "{}.{}".format(_jmb_component_name, _jmb_component_key),
+        )
+        processor.add_command(
+            InitialiseCommand(cexec_name, init_params),
+            end=True,
+        )
+        processor.add_command(
+            CallDisplayCommand(
+                cexec_name,
+                displayed_by_exec_name=self.exec_name,
+            ),
+            end=True,
+        )
+        processor.components_marked_for_removal = [
+            en for en in processor.components_marked_for_removal if en != cexec_name
+        ]
+
+    def remove_component(
+        self, _jmb_component_name: str, _jmb_component_key: Optional[str] = None
+    ):
+        """
+        Marks component for removal from user page (will not be displayed to the user)
+        without need to redisplay this entire component.
+
+        TODO: Check if component is rendered with {{component()}} and removed, if so
+        raise exception.
+        """
+        if _jmb_component_name not in self._config.components.keys():
+            raise JembeError(
+                "Component '{}' is not valid sub component of '{}'!".format(
+                    _jmb_component_name, self._config.full_name
+                )
+            )
+        processor = get_processor()
+        cexec_name = "{}/{}".format(
+            self.exec_name,
+            _jmb_component_name
+            if _jmb_component_key is None
+            else "{}.{}".format(_jmb_component_name, _jmb_component_key),
+        )
+        self.emit("_remove", removed_exec_name=cexec_name)
+        if cexec_name not in processor.components_marked_for_removal:
+            processor.components_marked_for_removal.append(cexec_name)
+
+    def get_storage(self, storage_name: Optional[str] = None) -> "jembe.Storage":
         processor = get_processor()
         return processor.jembe.get_storage(storage_name)
 
-    def redirect_to(self, component_ref: "ComponentReference"):
+    def redirect_to(self, component_ref: "jembe.ComponentReference"):
         """
         Redirects request to another component removing all unecessory
         components from processing.
@@ -1034,9 +1075,17 @@ class Component(metaclass=ComponentMeta):
                     self._jembe_disabled_actions.append(a)
 
     def ac_check(self, action_name: Optional[str] = None) -> bool:
+        """Check if access control rules are satisfied
+
+        Args:
+            action_name (Optional[str], optional): Check if action is accessible, when none it will check is whole component is accessible. Defaults to None.
+
+        Returns:
+            bool: returns true if action or whole compoennt is accesible
         """
-        Apply access control rules
-        """
-        if ComponentConfig.DEFAULT_DISPLAY_ACTION in self._jembe_disabled_actions:
+        # Apply access control rules
+        if ComponentConfig.DEFAULT_DISPLAY_ACTION in self._jembe_disabled_actions or (
+            action_name is not None and action_name in self._jembe_disabled_actions
+        ):
             return False
-        return action_name not in self._jembe_disabled_actions
+        return True
