@@ -1,7 +1,8 @@
 from typing import (
+    TYPE_CHECKING,
+    cast,
     Tuple,
     Union,
-    TYPE_CHECKING,
     TypeVar,
     Dict,
     List,
@@ -13,6 +14,7 @@ from typing import (
 )
 from abc import ABC, abstractmethod
 import re
+from json import JSONDecodeError
 from copy import deepcopy
 from enum import Enum
 from collections import deque
@@ -27,28 +29,30 @@ from lxml.html import Element
 from flask import json, jsonify, g
 from werkzeug import Response
 from .common import (
-    convert_to_annotated_type,
     exec_name_to_full_name,
     is_direct_child_name,
+    is_child_name,
     is_page_exec_name,
     parent_exec_name,
+    load_param,
+    # json_object_hook,
+    # json_default,
 )
-from .exceptions import JembeError
+from .exceptions import AccessDenied, Forbidden, JembeError, NotFound, Unauthorized
 from .component_config import ComponentConfig, RedisplayFlag as RedisplayFlag
 
 
 if TYPE_CHECKING:  # pragma: no cover
-    from .app import Jembe
+    import jembe
     from flask import Request
-    from .component import Component
-    from .component_config import ComponentListener
+    from jembe.component_config import ComponentListener
 
 
 class Event:
     def __init__(
         self,
         source_exec_name: str,
-        source: Optional["Component"],
+        source: Optional["jembe.Component"],
         event_name: str,
         to: Optional[str],
         params: dict,
@@ -58,7 +62,7 @@ class Event:
         self.params = params
         self.to: Optional[str] = to
 
-        self.source: Optional["Component"] = source
+        self.source: Optional["jembe.Component"] = source
 
     def __repr__(self):
         return "Event: source={}, name={}, to={} params={}".format(
@@ -102,7 +106,7 @@ TCommand = TypeVar("TCommand", bound="Command")
 
 
 class Command(ABC):
-    jembe: "Jembe"
+    jembe: "jembe.Jembe"
 
     def __init__(self, component_exec_name: str):
         self.component_exec_name = component_exec_name
@@ -175,7 +179,7 @@ class CallActionCommand(Command):
 
         # execute action
         action_result = getattr(component, self.action_name)(*self.args, **self.kwargs)
-        component.has_action_or_listener_executed = True
+        component._jembe_has_action_or_listener_executed = True
 
         # process action result
         if action_result is None or (
@@ -233,7 +237,12 @@ class CallActionCommand(Command):
 
 
 class CallDisplayCommand(CallActionCommand):
-    def __init__(self, component_exec_name: str, force: bool = False) -> None:
+    def __init__(
+        self,
+        component_exec_name: str,
+        force: bool = False,
+        displayed_by_exec_name: Optional[str] = None,
+    ) -> None:
         super().__init__(
             component_exec_name=component_exec_name,
             action_name=ComponentConfig.DEFAULT_DISPLAY_ACTION,
@@ -241,9 +250,24 @@ class CallDisplayCommand(CallActionCommand):
             kwargs=None,
         )
         self.force = force
+        self.displayed_by_exec_name = displayed_by_exec_name
+        self._displayed_components: List[str] = []
+
+    def mount(self, processor: "Processor") -> "CallDisplayCommand":
+        if self.displayed_by_exec_name:
+            if (
+                processor._processing_command
+                and isinstance(processor._processing_command, CallDisplayCommand)
+                and processor._processing_command.component_exec_name
+                == self.displayed_by_exec_name
+            ):
+                processor._processing_command._displayed_components.append(
+                    self.component_exec_name
+                )
+        return super().mount(processor)
 
     @cached_property
-    def _component(self) -> "Component":
+    def _component(self) -> "jembe.Component":
         return self.processor.components[self.component_exec_name]
 
     @cached_property
@@ -293,6 +317,7 @@ class CallDisplayCommand(CallActionCommand):
                 self._component.url,
                 self._component._config.changes_url,
                 action_result,
+                self._displayed_components,
             )
         elif isinstance(action_result, Response):
             # TODO If self.component is component directly requested via http request
@@ -341,7 +366,7 @@ class CallDisplayCommand(CallActionCommand):
         return commands
 
     def __repr__(self):
-        return "DisplayAction({}, {})".format(
+        return "CallDisplayCommand({}, {})".format(
             self.component_exec_name, self.action_name
         )
 
@@ -369,7 +394,7 @@ class CallListenerCommand(Command):
 
         # execute listener
         listener_result = getattr(component, self.listener_name)(self.event)
-        component.has_action_or_listener_executed = True
+        component._jembe_has_action_or_listener_executed = True
         if listener_result is None or (
             isinstance(listener_result, bool) and listener_result == True
         ):
@@ -392,8 +417,8 @@ class CallListenerCommand(Command):
             )
 
     def __repr__(self):
-        return "CallListener({},{})".format(
-            self.component_exec_name, self.listener_name
+        return "CallListener({}, {}, {})".format(
+            self.component_exec_name, self.listener_name, self.event
         )
 
 
@@ -457,7 +482,7 @@ class EmitCommand(Command):
             to=self._to,
             params=self.params,
         )
-        execute_over: List[Tuple["Component", str]] = []
+        execute_over: List[Tuple["jembe.Component", str]] = []
         for exec_name, component in self.processor.components.items():
             for (
                 listener_method_name,
@@ -668,23 +693,52 @@ class InitialiseCommand(Command):
         component_exec_name: str,
         init_params: dict,
         merge_existing_params: bool = True,
-        exist_on_client: bool = False,
+        exists_on_client: bool = False,
     ):
         super().__init__(component_exec_name)
-        self.init_params = {
+        self._init_params = {
             k: (v if not isinstance(v, Undefined) else None)
             for k, v in init_params.items()
         }
         self.merge_existing_params = merge_existing_params
-        self.exist_on_client = exist_on_client
+        self.exists_on_client = exists_on_client
+        self.displayed_components: List[str] = []
 
-        self.initialised_component: Optional["Component"] = None
-        self._cconfig: "ComponentConfig"
+        self.initialised_component: Optional["jembe.Component"] = None
+        self._cconfig: "jembe.ComponentConfig"
 
     def mount(self, processor: "Processor") -> "InitialiseCommand":
         self._cconfig = processor.jembe.get_component_config(self.component_exec_name)
         return super().mount(processor)
 
+    @cached_property
+    def existing_component(self) -> Optional["jembe.Component"]:
+        if self.component_exec_name in self.processor.components_marked_for_removal:
+            return None
+        return self.processor.components.get(self.component_exec_name, None)
+
+    @cached_property
+    def init_params(self) -> dict:
+        existing_params = (
+            {
+                k: v
+                for k, v in self.existing_component.state.items()
+                if k
+                not in self.existing_component.state._injected_params_names  # TODO: Test this and figure out why it's needed?
+            }
+            if self.merge_existing_params and self.existing_component
+            else dict()
+        )
+        init_params = (
+            {}
+            if self.merge_existing_params
+            else {**self._cconfig.component_class._jembe_state_param_default_values}
+        )
+        init_params.update(
+            {**existing_params, **self._init_params, **self._inject_into_params}
+        )
+        return init_params
+
     @cached_property
     def _inject_into_params(self) -> Dict[str, Any]:
         parent_cconfig = self._cconfig.parent
@@ -716,25 +770,32 @@ class InitialiseCommand(Command):
             return dict()
 
     def _must_do_init(self, is_accessible_run: bool):
-        if self.component_exec_name in self.processor.components:
-            component = self.processor.components[self.component_exec_name]
-            new_params = (
-                {}
-                if self.merge_existing_params
-                else {**self._cconfig.component_class._jembe_state_param_default_values}
-            )
-            new_params.update(
-                {**self.init_params, **self._inject_into_params, **component.inject()}
-            )
+        if self.existing_component:
+            # new_params = (
+            #     {}
+            #     if self.merge_existing_params
+            #     else {**self._cconfig.component_class._jembe_state_param_default_values}
+            # )
+            # new_params.update(
+            #     {
+            #         **self._init_params,
+            #         **self._inject_into_params,
+            #         **self.existing_component.inject(),
+            #     }
+            # )
             # if state params are same continue
             # else raise jembeerror until find better solution
             has_new_params = False
-            for k, v in new_params.items():
-                if k in component.state and v != component.state[k]:
+            # for k, v in new_params.items():
+            for k, v in self.init_params.items():
+                if (
+                    k in self.existing_component.state
+                    and v != self.existing_component.state[k]
+                ):
                     has_new_params = True
                     break
 
-            if component.has_action_or_listener_executed:
+            if self.existing_component._jembe_has_action_or_listener_executed:
                 if has_new_params:
                     if is_accessible_run:
                         return True
@@ -742,8 +803,14 @@ class InitialiseCommand(Command):
                         raise JembeError(
                             (
                                 "Cant reinitialise component {} with new parametes after "
-                                "action or listener is executed by this component."
-                            ).format(component.exec_name)
+                                "action or listener is executed by this component.\n"
+                                "Existing state: {}\n"
+                                "New state: {}"
+                            ).format(
+                                self.component_exec_name,
+                                self.existing_component.state,
+                                self.init_params,
+                            )
                         )
                 else:
                     return False
@@ -755,33 +822,13 @@ class InitialiseCommand(Command):
         # create new component if component with identical exec_name does not exist
         # or component with identical exec_name has not action or listener executed
         if self._must_do_init(is_accessible_run):
-            existing_component = self.processor.components.get(
-                self.component_exec_name, None
-            )
-            existing_params = (
-                {
-                    k: v
-                    for k, v in existing_component.state.items()
-                    if k in existing_component.state._injected_params_names
-                }
-                if self.merge_existing_params and existing_component
-                else dict()
-            )
-            init_params = (
-                {}
-                if self.merge_existing_params
-                else {**self._cconfig.component_class._jembe_state_param_default_values}
-            )
-            init_params.update(
-                {**existing_params, **self.init_params, **self._inject_into_params}
-            )
-
             component = self._cconfig.component_class._jembe_init_(
                 self._cconfig,
                 self.component_exec_name,
                 list(self._inject_into_params.keys()),
                 self.merge_existing_params,
-                **init_params
+                self.existing_component.state if self.existing_component else None,
+                **self.init_params
             )
 
             # check if action passes access control
@@ -792,7 +839,7 @@ class InitialiseCommand(Command):
             if not is_accessible_run:
                 self.processor.components[component.exec_name] = component
 
-                if self.exist_on_client:
+                if self.exists_on_client:
                     self.processor.renderers[component.exec_name] = ComponentRender(
                         False,
                         component.state.tojsondict(component, True),
@@ -801,6 +848,7 @@ class InitialiseCommand(Command):
                         component.url,
                         component._config.changes_url,
                         None,
+                        self.displayed_components,
                     )
 
                 if self.processor._emited_event_commands:
@@ -823,11 +871,14 @@ class InitialiseCommand(Command):
         return ()
 
     def __repr__(self):
-        return "Init({})".format(self.component_exec_name)
+        try:
+            return "Init({}, {})".format(self.component_exec_name, self.init_params)
+        except:
+            return "Init({}, {})".format(self.component_exec_name, self._init_params)
 
 
 class ComponentRender(NamedTuple):
-    """represents rendered coponent html with additional parametars"""
+    """Represents html rendered by component with additional parametars"""
 
     fresh: bool
     state_jsondict: Dict[str, Any]
@@ -836,10 +887,11 @@ class ComponentRender(NamedTuple):
     url: Optional[str]
     changes_url: bool
     html: Optional[str]
+    displayed_components: List[str]
 
 
 class CommandsQue:
-    def __init__(self, jembe: "Jembe") -> None:
+    def __init__(self, jembe: "jembe.Jembe") -> None:
         self.commands: Deque["Command"] = deque()
         self.deferred_commands: Deque["Command"] = deque()
 
@@ -860,7 +912,7 @@ class CommandsQue:
             raise JembeError("Cann't add unmounted command to commands que")
 
         def _do_add_command(que, command, end):
-            """ Adds command with before and after command to stack"""
+            """Adds command with before and after command to stack"""
             before_emit_commands = command.get_before_emit_commands()
 
             if end:
@@ -925,14 +977,17 @@ class Processor:
        (display) will not be called
     """
 
-    def __init__(self, jembe: "Jembe", component_full_name: str, request: "Request"):
+    def __init__(
+        self, _jembe: "jembe.Jembe", component_full_name: str, request: "Request"
+    ):
         g.jmb_processor = self
 
-        self.jembe = jembe
+        self.jembe = _jembe
         self.request = request
 
-        self.components: Dict[str, "Component"] = dict()
+        self.components: Dict[str, "jembe.Component"] = dict()
         self._commands: Deque["Command"] = deque()
+        self._processing_command: Optional["Command"] = None
         # already emited and processed event commands
         # that will be executed over newelly initialised component
         self._emited_event_commands: Deque["EmitCommand"] = deque()
@@ -941,12 +996,18 @@ class Processor:
         self._staging_commands = CommandsQue(self.jembe)
         # component renderers is dict[exec_name] = (componentState, url, rendered_str)
         self.renderers: Dict[str, "ComponentRender"] = dict()
+        # list of execnames marked for removal by jembe client js without redisplaying parent
+        self.components_marked_for_removal: List[str] = []
         # component that raised exception on initialise
         # any subsequent command on this component should be ignored
         self._raised_exception_on_initialise: Dict[str, dict] = dict()
         # direct response if component display returns it
         self._response: Optional["Response"] = None
 
+        # List of init commands without coresponding call command that
+        # need to be checked for redisplay depending of other redisplayed compoents
+        # on page (when processing x-jembe request)
+        self._hanging_init_commands_execnames: List[str] = []
         self.__create_commands(component_full_name)
         self._staging_commands.move_commands_to(self._commands)
 
@@ -966,20 +1027,18 @@ class Processor:
             )
         return load_params
 
-    def _x_jembe_command_factory(
-        self, command_data: dict, is_component: bool = False
-    ) -> "Command":
+    def _x_jembe_command_factory(self, command_data: dict) -> "Command":
         """Process data received by json and build commands
 
         is_component - mark that command_data is received in components section of x-jembe request
                     describing current compoents and their state displayed to the user
         """
 
-        if is_component:
+        if "execName" in command_data:
             return InitialiseCommand(
                 command_data["execName"],
                 self._load_init_params(command_data["execName"], command_data["state"]),
-                exist_on_client=True,
+                exists_on_client=True,
             )
         elif command_data["type"] == "init":
             return InitialiseCommand(
@@ -1015,24 +1074,49 @@ class Processor:
 
     def __create_commands(self, component_full_name: str):
         if self._is_x_jembe_request:
-            # x-jembe ajax request
-            data = json.loads(self.request.data)
-            # init components from data["components"]
-            to_be_initialised = []
-            for component_data in data["components"]:
-                self.add_command(
-                    self._x_jembe_command_factory(component_data, is_component=True),
-                    end=True,
+            try:
+                data = json.loads(
+                    self.request.data.decode("utf-8"),
                 )
-                to_be_initialised.append(component_data["execName"])
+            except JSONDecodeError:
+                data = dict()
+        if self._is_x_jembe_request and data:
+            # x-jembe ajax request
+            # init components from data["components"]
+            to_be_initialised = [
+                component_data["execName"] for component_data in data["components"]
+            ]
+            for component_data in data["components"]:
+                initcmd = cast(
+                    "InitialiseCommand", self._x_jembe_command_factory(component_data)
+                )
+                initcmd.displayed_components = [
+                    en
+                    for en in to_be_initialised
+                    if is_direct_child_name(initcmd.component_exec_name, en)
+                ]
+                self.add_command(initcmd, end=True)
             # init components from url_path if thay doesnot exist in data["compoenents"]
             self.__create_commands_from_url_path(component_full_name, to_be_initialised)
 
             # init commands
+            updated_init_commands_execnames: List[str] = []
+            call_commands_execnames: List[str] = []
             for command_data in data["commands"]:
-                self.add_command(self._x_jembe_command_factory(command_data), end=True)
+                command = self._x_jembe_command_factory(command_data)
+                self.add_command(command, end=True)
+                if isinstance(command, InitialiseCommand):
+                    updated_init_commands_execnames.append(command.component_exec_name)
+                elif isinstance(command, CallActionCommand):
+                    call_commands_execnames.append(command.component_exec_name)
+            # find all init commands without coresponding call or display command
+            self._hanging_init_commands_execnames = [
+                c
+                for c in updated_init_commands_execnames
+                if c not in call_commands_execnames
+            ]
         else:
-            # regular http/s GET request
+            # regular http/s GET request and when invalid x-jembe request data
             exec_names = self.__create_commands_from_url_path(
                 component_full_name, list()
             )
@@ -1075,12 +1159,15 @@ class Processor:
             if cconfig.name is None:
                 raise JembeError()
 
-            key = self.request.view_args[cconfig._key_url_param.identifier].lstrip(".")
+            request_view_args = (
+                self.request.view_args if self.request.view_args is not None else dict()
+            )
+            key = request_view_args[cconfig._key_url_param.identifier].lstrip(".")
             exec_name = Component._build_exec_name(cconfig.name, key, parent_exec_name)
             parent_exec_name = exec_name
             if exec_name not in to_be_initialised:
                 init_params = {
-                    up.name: self.request.view_args[up.identifier]
+                    up.name: request_view_args[up.identifier]
                     for up in cconfig._url_params
                 }
                 if (
@@ -1095,15 +1182,22 @@ class Processor:
                         value = self.request.args.get(url_param_name, None)
                         if value is not None:
                             try:
-                                init_params[
-                                    state_param_name
-                                ] = convert_to_annotated_type(
-                                    unquote_plus(self.request.args[url_param_name]),
+                                init_params[state_param_name] = load_param(
                                     cconfig.component_class._jembe_init_signature.parameters[
                                         state_param_name
-                                    ],
+                                    ].annotation,
+                                    unquote_plus(self.request.args[url_param_name]),
+                                )
+                            except ValueError as ve:
+                                current_app.logger.warning(
+                                    "{}: {} {}".format(
+                                        ve,
+                                        exec_name,
+                                        cconfig.component_class._jembe_init_signature.parameters[
+                                            state_param_name
+                                        ],
+                                    )
                                 )
-                            except ValueError:
                                 pass
 
                 self.add_command(
@@ -1153,6 +1247,52 @@ class Processor:
             self._staging_commands.move_commands_to(self._commands)
 
             self._execute_commands()
+            # for all components that have change state params but not have been redisplayed
+            # and all components with RedisplayFlag.WHEN_ON_PAGE:
+            # check will thay still be presented/visible on page if so execute display command
+            # for that components
+            for hanging_init_execname in self._hanging_init_commands_execnames + [
+                k
+                for k, v in self.components.items()
+                if RedisplayFlag.WHEN_ON_PAGE in v._config.redisplay
+            ]:
+                if hanging_init_execname not in [
+                    execname for execname, v in self.renderers.items() if v.fresh
+                ]:
+                    # if hanging init is displayed on page it should be rendered
+                    en_split = hanging_init_execname.split("/")
+                    execname_tree = []
+                    for i in range(2, len(en_split) + 1):
+                        execname_tree.append("/".join(en_split[:i]))
+                    # check if root component will be redisplayed
+                    root_fresh_keys = [
+                        k
+                        for k, v in self.renderers.items()
+                        if v.fresh and len(k.split("/")) == 2
+                    ]
+                    displayed_execnames = (
+                        root_fresh_keys
+                        if len(root_fresh_keys) > 0
+                        else [execname_tree[0]]
+                    )
+                    execute_display = True
+                    for ename in execname_tree:
+                        if ename not in displayed_execnames:
+                            execute_display = False
+                            break
+                        displayed_execnames = self.renderers[ename].displayed_components
+                    if execute_display:
+                        self.add_command(CallDisplayCommand(hanging_init_execname))
+                        self._staging_commands.move_commands_to(self._commands)
+                        self._execute_commands()
+                    elif (
+                        hanging_init_execname in self.components
+                        and RedisplayFlag.WHEN_ON_PAGE
+                        in self.components[hanging_init_execname]._config.redisplay
+                        and hanging_init_execname
+                        not in self.components_marked_for_removal
+                    ):
+                        self.components_marked_for_removal.append(hanging_init_execname)
             return self
 
         finally:
@@ -1183,7 +1323,9 @@ class Processor:
                 return None
 
         try:
+            self._processing_command = command
             response = command.execute()
+            self._processing_command = None
             self._staging_commands.move_commands_to(self._commands)
             if response is not None:
                 return response
@@ -1204,8 +1346,8 @@ class Processor:
             return None
 
     def execute_initialise_command_successfully(
-        self, command: "InitialiseCommand", *additional_components: "Component"
-    ) -> Tuple[bool, Optional["Component"]]:
+        self, command: "InitialiseCommand", *additional_components: "jembe.Component"
+    ) -> Tuple[bool, Optional["jembe.Component"]]:
         """
         Directly out of commands que execute initialise command
         in order to check will it raise Exception
@@ -1214,14 +1356,16 @@ class Processor:
         """
         # check if component with requested full_name exist or if
         # initialisation with same init_params already failed
-        if exec_name_to_full_name(
-            command.component_exec_name
-        ) not in self.jembe.components_configs or (
-            command.component_exec_name in self._raised_exception_on_initialise
-            and (
-                command.init_params
-                == self._raised_exception_on_initialise[command.component_exec_name]
-            )
+
+        if (
+            exec_name_to_full_name(command.component_exec_name)
+            not in self.jembe.components_configs
+        ):
+            return (False, None)
+        command = command if command.is_mounted else command.mount(self)
+        if command.component_exec_name in self._raised_exception_on_initialise and (
+            command.init_params
+            == self._raised_exception_on_initialise[command.component_exec_name]
         ):
             return (False, None)
 
@@ -1233,37 +1377,38 @@ class Processor:
         for acomp in additional_components:
             self.components[acomp.exec_name] = acomp
 
-        cmd: "InitialiseCommand" = (
-            command if command.is_mounted else command.mount(self)
-        )
         try:
-            cmd.execute(is_accessible_run=True)
+            command.execute(is_accessible_run=True)
         except JembeError as jmb_error:
             # JembeError are exceptions raised by jembe
             # and thay indicate bad usage of framework
             self.components = backup_current_components
             raise jmb_error
         except Exception as exc:
-            self._raised_exception_on_initialise[cmd.component_exec_name] = deepcopy(
-                cmd.init_params
-            )
+            self._raised_exception_on_initialise[
+                command.component_exec_name
+            ] = deepcopy(command.init_params)
             # initalise command is not run properly
 
             # restore _staging_commands
             self._staging_commands = backup_current_staging_commands
             self.components = backup_current_components
             if current_app.debug or current_app.testing:
-                # import traceback
-                # traceback.print_exc()
                 current_app.logger.warning(
                     "Exception when initialising component out of proccessing que {}: {}".format(
-                        cmd, exc.__repr__()
+                        command, exc.__repr__()
                     )
                 )
+                if not isinstance(
+                    exc, (AccessDenied, NotFound, Unauthorized, Forbidden)
+                ):
+                    import traceback
+
+                    traceback.print_exc()
             return (False, None)
         self._staging_commands = backup_current_staging_commands
         self.components = backup_current_components
-        return (True, cmd.initialised_component)
+        return (True, command.initialised_component)
 
     def _handle_exception_in_command(self, command: "Command", exc: "Exception"):
         """
@@ -1315,24 +1460,44 @@ class Processor:
             cmd = exception_commands.pop()
             # if exception is raised during handling exception ... dont try to catch it :)
             cmd.execute()
-            self._staging_commands.move_commands_to(exception_commands)
+            if emit_command.event.params["handled"]:
+                if isinstance(command, InitialiseCommand):
+                    # save initialises command that raised exception with params
+                    # so we can skip executing other commants on this component
+                    # (save to skip execution because we handled exception)
+                    self._raised_exception_on_initialise[
+                        command.component_exec_name
+                    ] = deepcopy(command.init_params)
+                    # remove all child compoennt of command.component_exec_name
+                    self.components = {
+                        exec_name: comp
+                        for exec_name, comp in self.components.items()
+                        if not is_child_name(cmd.component_exec_name, exec_name)
+                    }
+                    # remove all commands to and from  removed components
+                    self._commands = deque(
+                        (
+                            c
+                            for c in self._commands
+                            if not is_child_name(
+                                cmd.component_exec_name, c.component_exec_name
+                            )
+                        )
+                    )
+                exception_commands = deque()
+                self._staging_commands.move_commands_to(self._commands)
+                return
+            else:
+                self._staging_commands.move_commands_to(exception_commands)
 
-        if not emit_command.event.params["handled"]:
-            # if exception is not handled by parent components
-            # rerise exception
-            current_app.logger.error(
-                "Unhandled exception in {}: {}".format(command, exc)
-            )
-            raise emit_command.event.params["exception"]
+        # if exception is not handled by parent components
+        # rerise exception
+        current_app.logger.error("Unhandled exception in {}: {}".format(command, exc))
+        if current_app.debug or current_app.testing:
+            import traceback
 
-        # if exception is handled dont raise exception
-        if isinstance(command, InitialiseCommand):
-            # save initialises command that raised exception with params
-            # so we can skip executing other commants on this component
-            # (save to skip execution because we handled exception)
-            self._raised_exception_on_initialise[
-                command.component_exec_name
-            ] = deepcopy(command.init_params)
+            traceback.print_exc()
+        raise emit_command.event.params["exception"]
 
     def build_response(self) -> "Response":
         if self._response:
@@ -1350,9 +1515,10 @@ class Processor:
                     url,
                     changes_url,
                     html,
+                    components,
                 ),
             ) in self.renderers.items():
-                if fresh:
+                if fresh and exec_name not in self.components_marked_for_removal:
                     ajax_responses.append(
                         dict(
                             execName=exec_name,
@@ -1373,6 +1539,13 @@ class Processor:
                             },
                         )
                     )
+            if self.components_marked_for_removal:
+                ajax_responses.append(
+                    dict(
+                        globals=True,
+                        removeComponents=self.components_marked_for_removal,
+                    )
+                )
             return jsonify(ajax_responses)
         else:
             # for page with components build united response
@@ -1397,11 +1570,13 @@ class Processor:
                     url,
                     changes_url,
                     html,
+                    components,
                 ) in self.renderers.items()
                 if fresh
                 and state_jsondict is not None
                 and url is not None
                 and html is not None
+                and exec_name not in self.components_marked_for_removal
             }
             unused_exec_names = sorted(
                 c_etrees.keys(),
@@ -1416,24 +1591,42 @@ class Processor:
                 # compose response including all components not just page
                 # find all placeholders in response_tree and replace them with
                 # appropriate etrees
+                placeholders = []
+                permanent_placeholders = []
                 for placeholder in response_etree.xpath(
-                    ".//template[@jmb-placeholder]"
+                    ".//template[@jmb-placeholder or @jmb-placeholder-permanent]"
                 ):
-                    can_find_placeholder = True
-                    exec_name = placeholder.attrib["jmb-placeholder"]
+                    if "jmb_placeholder" in placeholder.attrib:
+                        placeholders.append(placeholder)
+                    else:
+                        permanent_placeholders.append(placeholder)
+
+                for placeholder in placeholders + permanent_placeholders:
                     try:
-                        c_etree = c_etrees[exec_name]
-                        unused_exec_names.pop(unused_exec_names.index(exec_name))
-                        placeholder.addnext(c_etree)
-                        placeholder.getparent().remove(placeholder)
+                        permanent = False
+                        exec_name = placeholder.attrib["jmb-placeholder"]
                     except KeyError:
-                        # exec_name referenced by this placeholder does not exist
-                        # so we will just remove placeholder
-                        # This situation can ocure when handling exceptions
-                        # of child components but not changing display html
-                        # so we can assume that developer just want to ignore
-                        # exception and dont display component that coused exception
-                        placeholder.getparent().remove(placeholder)
+                        permanent = True
+                        exec_name = placeholder.attrib["jmb-placeholder-permanent"]
+
+                    if exec_name in unused_exec_names:
+                        can_find_placeholder = True
+                        unused_exec_names.pop(unused_exec_names.index(exec_name))
+                        try:
+                            c_etree = c_etrees[exec_name]
+                            placeholder.addnext(c_etree)
+                        except KeyError:
+                            # exec_name referenced by this placeholder does not exist
+                            # so we will just remove placeholder
+                            # This situation can ocure when handling exceptions
+                            # of child components but not changing display html
+                            # so we can assume that developer just want to ignore
+                            # exception and dont display component that coused exception
+                            pass
+                        finally:
+                            if not permanent:
+                                placeholder.getparent().remove(placeholder)
+
             # Remove empty placeholder if thay are left in response
             # because above logic will not find all empty placeholders
             if response_etree is not None:
@@ -1477,10 +1670,11 @@ class Processor:
                     ),
                     separators=(",", ":"),
                     sort_keys=True,
+                    # default=json_default
                 ),
             )
 
-        if not html:
+        if not html.strip():
             html = "<div></div>"
         root = etree.HTML(html)
         if is_page_exec_name(exec_name):
@@ -1509,14 +1703,15 @@ class Processor:
     @cached_property
     def _is_x_jembe_request(self) -> bool:
         return bool(
-            self.request.headers.has_key(self.jembe.X_JEMBE)
+            self.jembe.X_JEMBE in self.request.headers
             and self.request.headers.get(self.jembe.X_JEMBE) != "upload"
         )
 
     @cached_property
     def _is_x_jembe_upload_followup_request(self) -> bool:
-        return self._is_x_jembe_request and self.request.headers.has_key(
-            self.jembe.X_RELATED_UPLOAD
+        return (
+            self._is_x_jembe_request
+            and self.jembe.X_RELATED_UPLOAD in self.request.headers
         )
 
     @cached_property
