@@ -1077,25 +1077,25 @@ def test_refresh_task_list_x(jmb, client):
     )
     assert json_response[0]["dom"] == (
         "<div>"
-        """<button type="button" jmb-on:click="$jmb.component('add').display()">Add</button>"""
+        """<button type="button" jmb-on:click="$jmb.component(\'add\').display()">Add</button>"""
         "<table>"
         "<tr><th>Task</th><th>Actions</th></tr>"
         "<tr>"
         "<td>"
-        """<a href="/tasks/tasks/view/1" jmb-on:click="$jmb.component('view',{task_id:1}).display()">Task 1</a>"""
+        """<a href="/tasks/tasks/view/1" jmb-on:click="$jmb.component(\'view\',{\'task_id\':1}).display()">Task 1</a>"""
         "</td>"
         "<td>"
-        """<a href="/tasks/tasks/edit/1" jmb-on:click="$jmb.component('edit',{task_id:1}).display()">edit</a>"""
-        """<a href="/tasks/tasks/delete/1" jmb-on:click="$jmb.component('delete',{task_id:1}).display()">delete</a>"""
+        """<a href="/tasks/tasks/edit/1" jmb-on:click="$jmb.component(\'edit\',{\'task_id\':1}).display()">edit</a>"""
+        """<a href="/tasks/tasks/delete/1" jmb-on:click="$jmb.component(\'delete\',{\'task_id\':1}).display()">delete</a>"""
         "</td>"
         "</tr>"
         "<tr>"
         "<td>"
-        """<a href="/tasks/tasks/view/2" jmb-on:click="$jmb.component('view',{task_id:2}).display()">Task 2</a>"""
+        """<a href="/tasks/tasks/view/2" jmb-on:click="$jmb.component(\'view\',{\'task_id\':2}).display()">Task 2</a>"""
         "</td>"
         "<td>"
-        """<a href="/tasks/tasks/edit/2" jmb-on:click="$jmb.component('edit',{task_id:2}).display()">edit</a>"""
-        """<a href="/tasks/tasks/delete/2" jmb-on:click="$jmb.component('delete',{task_id:2}).display()">delete</a>"""
+        """<a href="/tasks/tasks/edit/2" jmb-on:click="$jmb.component(\'edit\',{\'task_id\':2}).display()">edit</a>"""
+        """<a href="/tasks/tasks/delete/2" jmb-on:click="$jmb.component(\'delete\',{\'task_id\':2}).display()">delete</a>"""
         "</td>"
         "</tr>"
         "</table>"
